@@ -1,5 +1,6 @@
 #define _GNU_SOURCE
 #include <stdio.h>
+#include <magic.h>
 #include <string.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -23,6 +24,7 @@
 #include "config.h"
 #include <fuse.h>
 
+
 #define RHEL5_LIBCURL_VERSION 462597
 #define RHEL5_CERTIFICATE_FILE "/etc/pki/tls/certs/ca-bundle.crt"
 
@@ -512,11 +514,44 @@ void cloudfs_init()
   }
 }
 
+int file_is_readable(const char *fname)
+{
+    FILE *file;
+    if ( file = fopen( fname, "r" ) )
+    {
+        fclose( file );
+        return 1;
+    }
+    return 0;
+}
+
+const char * get_file_mimetype ( const char *path )
+{
+    if( file_is_readable( path ) == 1 )
+    {
+      magic_t magic;
+      const char *mime;
+
+      magic = magic_open( MAGIC_MIME_TYPE );
+      magic_load( magic, NULL );
+      magic_compile( magic, NULL );
+      mime = magic_file( magic, path );
+      magic_close( magic );
+
+      return mime;
+  }
+
+  const char *error = "application/octet-stream";
+  return error;
+}
+
+
 int cloudfs_object_read_fp(const char *path, FILE *fp)
 {
 
   long flen;
   fflush(fp);
+  const char *filemimetype = get_file_mimetype( path );
 
   // determine the size of the file and segment if it is above the threshhold
   fseek(fp, 0, SEEK_END);
@@ -574,6 +609,7 @@ int cloudfs_object_read_fp(const char *path, FILE *fp)
     add_header(&headers, "x-object-manifest", manifest);
     add_header(&headers, "x-object-meta-mtime", meta_mtime);
     add_header(&headers, "Content-Length", "0");
+    add_header(&headers, "Content-Type", filemimetype);
 
     int response = send_request_size("PUT", encoded, NULL, NULL, headers, 0, 0);
     curl_slist_free_all(headers);
